<?php
namespace Teknisa\EST\Modules\EstoqueCego\Controllers;

use Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use \Teknisa\Libs\Util\DateUtil;
use \Geral\Entities\Paramfilial;

class Est96000DivergenciasInventarioCegoController extends TeknisaCrud {

    protected $dataSourceName = "";
    protected $parameterBagColumns = [];
    protected $dataSourceManager;
    protected $environment;
    protected $crudService;
    protected $service;
    protected $newCode;
    
    protected $nrorg;
    protected $currentTime;
    protected $operador;
    protected $filialLogada;
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $validacoesEstoqueService,$Est96000DivergenciasInventarioCegoService,$newCode) {
        $this->crudService = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->service = $Est96000DivergenciasInventarioCegoService;
        $this->newCode = $newCode;
        
        $this->operador = $environment->getCdOperador();
        $this->nrorg = $environment->getNrOrg();
        $this->filialLogada = $environment->getCdFilial();
        $this->currentTime = $environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->environment = $environment;
    }
    
    public function find(Filter $request, Response $response) {
        
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
                
            // get params on each position
            foreach($conditions as $condition)
                $paramBag[$condition['columnName']] = $condition['value'];
                
            // get items row
            $this->dataSourceName = 'iteminventcego_produto';
            $newFilterCriteria = new FilterCriteria($this->dataSourceName, []);
            
            $newFilterCriteria->addCondition('CDFILIAL', $paramBag['CDFILIAL']);
            $newFilterCriteria->addCondition('CDALMOXARIFE', $paramBag['CDALMOXARIFE']);
            $newFilterCriteria->addCondition('CDLOCALESTOQ', $paramBag['CDLOCALESTOQ']);
            $newFilterCriteria->addCondition('DTCONTAGEM', $paramBag['DTCONTAGEM']);
            
            $filter = new DTO\Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::find($filter, $response);
            
            $ItemsRows = $response->getDataSets()[0]->getRows();
            
            //validate each row
            if(!empty($ItemsRows)){
                
                // get filial param cost type
                $newFilterCriteria = new FilterCriteria("paramfilial", []);
                $newFilterCriteria->addCondition('CDFILIAL', $paramBag['CDFILIAL']);
                $newFilterCriteria->addCondition('NRORG', $this->environment->getNrOrgTrab());
                $newFilterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($newFilterCriteria);
                
                $this->dataSourceName = 'paramfilial';
                $filter = new DTO\Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                
                $newResponse = new Response();
                parent::find($filter, $newResponse);
                
                $UnitParams = $newResponse->getDataSets()[0]->getRows()[0];
                
                //add one day on DTPROCESSA 
                $datetime = DateUtil::getDataDeString(substr($paramBag['DTCONTAGEM'],0,10));
                $datetime->modify('+1 day');
                $NextDay = $datetime->format('d-m-Y');
                
                // set withdraw date according to IDORDAJUINVE param
                $UnitParams['DTRETI'] = $UnitParams['IDORDAJUINVE'] == 'U' ? $NextDay : $paramBag['DTCONTAGEM'];
                
                //get cost type
                $CostType = !empty($UnitParams['CDTIPCUSTEST']) ? $UnitParams['CDTIPCUSTEST'] : 3;
                
                //builds description Date field
                $Contagem = substr($paramBag['DTCONTAGEM'],0,10);
                $Geracao = substr($paramBag['DTGERACAOINVEST'],0,10);
                $Aprovacao = substr($paramBag['DTAPROVINVESTCE'],0,10);
                
                $descriptionDates = $Contagem ." - Data de Geração: ".$Geracao." - Data de Aprovação: ".$Aprovacao;
                
                $items = [
                    'writeOffItems' => [],
                    'ControlItems' => []
                ];
                
                foreach($ItemsRows as $row){
                    //checks stock position
                    $filial = $row['CDFILIAL'];
                    $tipoCusto = $CostType;
                    $produto = $row['CDPRODUTO'];
                    $almoxarifado = $row['CDALMOXARIFE'];
                    $localEstoque = $row['CDLOCALESTOQ'];
                    $lote = $row['NRLOTEESTQ'];
                    $subLote = $row['NRSUBLOTE'];
                    $dtContagem = $row['DTCONTAGEM'];
                    
                    // data about cost and quantity of the actual product
                    $custoQuantidade = $this->validacoesEstoqueService->consultaCustoQuantidadeProduto(
                        $filial, $produto, $dtContagem, $tipoCusto, $almoxarifado, $localEstoque, $lote, $subLote
                    );
                    
                    // gets percentage according to qty from real stock and blind stock
                    $estCegoPercent = $custoQuantidade['QTESTOQDIA'] == 0 ? 0 : ((($row['QTINVENTESTCEGO'] * 100)/$custoQuantidade['QTESTOQDIA']) - 100);
                    
                    
                    // gets blind stock percentage divergence 
                    $PercDivBlindStock = $this->service->getParperdivestceg($filial,$produto);
                    
                    if($estCegoPercent >= $PercDivBlindStock) {
                        if(abs($row['QTINVENTESTCEGO'] - $custoQuantidade['QTESTOQDIA']) > 0.00001) {
                            $row['QTDIVER'] = $row['QTINVENTESTCEGO'] - $custoQuantidade['QTESTOQDIA'];
                            $row['VRDIVER'] = $custoQuantidade['VRCUSTOPROD'] * $row['QTDIVER'];
                            $row['DESCRIPTIONDATES'] = $descriptionDates;
                            
                            if($row['QTINVENTESTCEGO'] > $custoQuantidade['QTESTOQDIA']) {
                                $items['ControlItems'][] = $row;
                            }else 
                                $items['writeOffItems'][] = $row;
                        }
                    }
                }
            
            }else throw new \Exception("Não há dados a serem exibidos.",404);
            
            // returns dataSet according to param MODE
            switch($paramBag['MODE']){
                case "B":
                    $response->setDataSets(array(new DataSet('getItemsDivergencia', $items['writeOffItems'])));
                    break;
                case "C":
                    $response->setDataSets(array(new DataSet('getItemsDivergencia', $items['ControlItems'])));
                    break;
                default:
                    $response->setDataSets(array(new DataSet('getItemsDivergencia', $items)));
                    break;
            }
                
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function actionsManagementHandler(Request $request, Response $response) {
        try {
            
            $rows = $request->getDataSet()->getRows();
            $params = $request->getParameters();
            $cdfilimovi = $rows[0]["CDFILIAL"];
            $cdalmoxarife = $rows[0]["CDALMOXARIFE"];
            $cdlocalestoq = $rows[0]["CDLOCALESTOQ"];
            $dtcontagem = $rows[0]["DTCONTAGEM"];
            
            switch($params['action']){
                case 'accept':{
                    if($params['hasItems']){
                        // get params stock
                        $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($cdfilimovi, $this->nrorg);
            
                        $cdservbaiestceg = $paramfilial->getCdservbaiestceg();
                        $cdtipcustest = $paramfilial->getCdtipcustest();
                        $idordajuinve = $paramfilial->getIdordajuinve();
                        $cdtpretbxestceg = $paramfilial->getCdtpretbxestceg();
                        
                        if(!empty($cdservbaiestceg)){
                            //gets cost type
                            $CostType = !empty($cdtipcustest) ? $cdtipcustest : 3;
                            
                             //adds one day on DTPROCESSA 
                            $datetime = DateUtil::getDataDeString(substr($dtcontagem,0,10));
                            $datetime->modify('+1 day');
                            $NextDay = $datetime->format('d-m-Y');
                            
                            // sets withdraw date according to IDORDAJUINVE param
                            $dtreti = $idordajuinve == 'U' ? $NextDay : $dtcontagem;
                            
                            //get cardapio info
                            $CardapioInfo = $this->service->getCardapioInfo($cdfilimovi,$dtreti,$cdservbaiestceg);
                            if(empty($CardapioInfo)){
                                $NrCardapio = $this->geraNewSequencial('CARDADIA', $cdfilimovi);
                                //insert new cardapio
                                $newCadapioData = [
                                    'CDFILIAL' => $cdfilimovi,
                                    'NRCARDAPIO' => $NrCardapio,
                                    'CDSERVFILI' => $cdservbaiestceg,
                                    'CDOPERADOR' => $this->operador,
                                    'DTCARDAPIO' => $dtreti,
                                    'DTMESCARD'  => $dtreti,
                                    "NRORG"         =>  $this->environment->getNrOrgTrab(),
                                    "__is_new"      =>  true
                                ];
                                
                                $this->dataSourceName = 'cardadia';
                                $newRequest = new DTO\Request\Row ($newCadapioData, '', '', '');
                                parent::save($newRequest, $response);
                                
                            }else $NrCardapio = $CardapioInfo[0]['NRCARDAPIO'];
                            
                            //Inserts into lanctoestq
                            $nrlancestq = $this->geraNewSequencial('lanctoestoq', 'NRLANCESTQ');
                            
                            $lanctoestData = [
                                'CDFILIAL'    => $this->filialLogada,
                                'NRLANCESTQ'  => $nrlancestq,
                                'IDTPLANCTO'  => 3, 
                                'DTLANCESTQ'  => $dtreti,
                                'CDFILIMOVI'  => $cdfilimovi,
                                'CDOPERADOR'  => $this->operador,
                                'IDALTCUSMED' => 'N',
                                'DSLANCESTQ'  => '',
                                "NRORG"       => $this->nrorg,
                                "__is_new"    => true
                            ];
                            $this->dataSourceName = 'lanctoestoq';
                            $newRequest = new DTO\Request\Row ($lanctoestData, '', '', '');
                            parent::save($newRequest, $response);
                            
                            //Inserts into retirada
                            $NrRetirada = $this->geraNewSequencial('RETIESTO', 'NRRETIRADA');
                            
                            $retiradaData = [
                                "CDFILIAL"     => $cdfilimovi,
                                "NRRETIRADA"   => $NrRetirada,
                                "NRCARDAPIO"   => $NrCardapio,
                                "DTRETIRADA"   => $dtreti,
                                "CDFILANCESTQ" => $this->filialLogada,
                                'NRLANCESTQ'   => $nrlancestq,
                                "NRORG"       => $this->nrorg,
                                "__is_new"    => true
                            ];
                            $this->dataSourceName = 'retirada';
                            $newRequest = new DTO\Request\Row ($retiradaData, '', '', '');
                            parent::save($newRequest, $response);
                            
                            
                            //Builds items params for itlanctoest
                            
                            foreach($rows as $item){
                                $Nrseqitem = $this->geraNewSequencial('ITEMRETI', 'NRSEQUITRETI',4);
                                
                                unset($item['__groupHeaders']);
                                
                                // Gets product Info
                                $newFilterCriteria = new FilterCriteria("produto", []);
                                $newFilterCriteria->addCondition('CDPRODUTO', $item['CDPRODUTO']);
                                $newFilterCriteria->addCondition('NRORG', $this->nrorg);
                                $newFilterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($newFilterCriteria);
                                
                                $this->dataSourceName = 'produto';
                                $filter = new DTO\Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                                
                                $newResponse = new Response();
                                parent::find($filter, $newResponse);
                                
                                $productInfo = $newResponse->getDataSets()[0]->getRows()[0];
                                
                                //checks stock position
                                $filial = $item['CDFILIAL'];
                                $tipoCusto = $CostType;
                                $produto = $item['CDPRODUTO'];
                                $almoxarifado = $item['CDALMOXARIFE'];
                                $localEstoque = $item['CDLOCALESTOQ'];
                                $lote = $item['NRLOTEESTQ'];
                                $subLote = $item['NRSUBLOTE'];
                                $dtContagem = $item['DTCONTAGEM'];
                                
                                // data about cost and quantity of the actual product
                                $posestdiaInfo = $this->validacoesEstoqueService->consultaCustoQuantidadeProduto(
                                    $filial, $produto, $NextDay, $tipoCusto, $almoxarifado, $localEstoque, $lote, $subLote
                                );
                                
                                $item['CDFILIMOVI'] = $cdfilimovi;
                                $item['CDFILIAL']   = $this->filialLogada;
                                $item['CDPRODMOVI'] = $productInfo['CDPRODESTO'];
                                $item['NRLANCESTQ']  = $nrlancestq;
                                $item['NRSEQUITEM'] = $Nrseqitem;
                                $item['NRSEQUITRETI'] = $Nrseqitem;
                                $item['IDTIPOMOVI'] = '3';
                                $item['IDEVOLESTOQ'] = 'S';
                                $item['IDORDEMCLC'] = '9';
                                $item['IDALTCMEDMV'] = 'N';
                                $item['CDTIPORETI'] = $cdtpretbxestceg;
                                $item['QTLANCTOEST'] = ($posestdiaInfo['QTESTOQDIA'] - $item['QTINVENTESTCEGO']) * $productInfo['VRFATOCONV'];
                                $item['QTTOTLANCTO'] = $item['QTLANCTOEST'];
                                $item['VRUNILANCTO'] = $posestdiaInfo['VRCUSTOPROD'] * $productInfo['VRFATOCONV'];
                                $item['VRTOTLANCTO'] = $item['VRUNILANCTO'] * $item['QTTOTLANCTO'];
                                $item['NRRETIRADA'] = $NrRetirada;
                                $item['NRCARDAPIO'] = $NrCardapio;
                                $item['QTRETIRADA'] = $item['QTLANCTOEST'];
                                $item['__is_new'] = true;
                            }
                            
                            //Inserts into itlanctoest
                            $newDatasetToSave = new DataSet('itlanctoest', $rows);
                            $newRequestToSave = new DTO\Request\DataSet($newDatasetToSave, '', '', '');
                            $CompletedDataset = $this->crudService->getRowsToSave($newRequestToSave, "itlanctoest");
                            $dataSetToSave = new DataSet("itlanctoest", $CompletedDataset);
                            $this->dataSourceManager->persist($dataSetToSave);
                            
                            //Complete items params for itemreti
                            foreach($rows as $item){
                                $item['CDFIITLANEST'] = $item['CDFILIMOVI'];
                                $item['CDPRODUTO']    = $item['CDPRODMOVI'];
                            }
                            
                            //Inserts into itemreti
                            $newDatasetToSave = new DataSet('itemreti', $rows);
                            $newRequestToSave = new DTO\Request\DataSet($newDatasetToSave, '', '', '');
                            $CompletedDataset = $this->crudService->getRowsToSave($newRequestToSave, "itemreti");
                            $dataSetToSave = new DataSet("itemreti", $CompletedDataset);
                            $this->dataSourceManager->persist($dataSetToSave);
                            
                            //updates Stock deleting irregular items 
                            $this->validacoesEstoqueService->atualizaEstoque($cdfilimovi, $nrlancestq,false,true,false);
                            
                            //updates blind inventory
                            $blindInventoryParams = [
                                "CDOPERADOR"      => $this->operador,
                                "DTRECACEITAJUST" => $this->currentTime,
                                "DSOBSRECAJUST"   => '',
                                "IDSITUACEITEINV" => 'A',
                                "CDFILIAL"        => $cdfilimovi,
                                'CDALMOXARIFE'    => $cdalmoxarife,
                                "CDLOCALESTOQ"    => $cdlocalestoq,
                                "DTCONTAGEM"      => $dtcontagem,
                                "NRORG"           => $this->nrorg,
                                "__is_new"        => false
                            ];
                            
                            $this->dataSourceName = 'inventestcego';
                            $newRequest = new DTO\Request\Row ($blindInventoryParams, '', '', '');
                            parent::save($newRequest, $response);
                        }else{
                            throw new \Exception("Não existe serviço parametrizado para a baixa automática.");
                        }
                    }else{
                        $blindInventoryParams = [
                            "CDOPERADOR"      => $this->operador,
                            "DTRECACEITAJUST" => $this->currentTime,
                            "DSOBSRECAJUST"   => '',
                            "IDSITUACEITEINV" => 'A',
                            "CDFILIAL"        => $cdfilimovi,
                            'CDALMOXARIFE'    => $cdalmoxarife,
                            "CDLOCALESTOQ"    => $cdlocalestoq,
                            "DTCONTAGEM"      => $dtcontagem,
                            "NRORG"           => $this->nrorg,
                            "__is_new"        => false
                        ];
                        
                        $this->dataSourceName = 'inventestcego';
                        $newRequest = new DTO\Request\Row ($blindInventoryParams, '', '', '');
                        parent::save($newRequest, $response);
                    }
                }
                    break;
                case 'refuse':{
                    $blindInventoryParams = [
                        "CDOPERADOR"      => $this->operador,
                        "DTRECACEITAJUST" => $this->currentTime,
                        "DSOBSRECAJUST"   => '',
                        "IDSITUACEITEINV" => 'R',
                        "CDFILIAL"        => $cdfilimovi,
                        'CDALMOXARIFE'    => $cdalmoxarife,
                        "CDLOCALESTOQ"    => $cdlocalestoq,
                        "DTCONTAGEM"      => $dtcontagem,
                        "NRORG"           => $this->nrorg,
                        "__is_new"        => false
                    ];
                    $this->dataSourceName = 'inventestcego';
                    $newRequest = new DTO\Request\Row ($blindInventoryParams, '', '', '');
                    parent::save($newRequest, $response);
                }
                    break;
                case 'adjust':
                    //pendendte implementacao
                    break;
                default:
                    break;
            }
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function geraNewSequencial($table, $param = '', $colSize = 10) {
        return $this->newCode->getCodeByTableName($table.$param, $this->nrorg, 1, $colSize);
    }
    
}